//     

#define F_CPU 1200000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>


/*******   *******/
#define  MOTOR_LEFT    10
#define  MOTOR_RIGHT   20
#define  MOTOR_STOP    30
#define  MOTOR_START   40
#define  RESET         0
/****************************/


#define REL_PWR     PB0       //   
#define REL_RVS     PB2       //      
#define LED_OUT     PB4       //   
#define LED_IN      PB3       //   
#define BUTTON      PB1       //  



/* 1 = 100 */
#define  MOTOR_DELAY     5       // :     
#define  TIME_CYCLE      300     //  ,  : MOTOR_DELAY*4

#define  WORK_COUNT      5       //   
 
#define  BUTTON_LDELAY   16      //:   
#define  BUTTON_SDELAY   4       //:   



volatile unsigned char mprog_select  = 0;
volatile unsigned char motor_mode    = 0;
volatile unsigned char motor_timer   = 0;
volatile unsigned char button_timer  = 0;
volatile unsigned int  cycle_timer   = 0;
volatile unsigned char work_cnt      = 0;
		 unsigned char motor_dir     = 0;



int main(void)
{
	/*  */
	DDRB  &= ~(1<<BUTTON);                                        // 
	PORTB |=  (1<<BUTTON);                                        // 
	/*  */
	DDRB  |=  (1<<REL_PWR)|(1<<REL_RVS)|(1<<LED_OUT)|(1<<LED_IN); // 
	PORTB &= ~((1<<REL_PWR)|(1<<REL_RVS)|(1<<LED_OUT));           // 
	PORTB |=  (1<<LED_IN);

	
	TCCR0B = (1<<CS02)|(0<<CS01)|(1<<CS00);                       //1171,875Hz
	TIMSK0 = (1<<OCIE0A);										  //  
	OCR0A = 116;                                                  // ~10Hz


	sei();//    


	
	while (1) 
	{

		if (!(PINB & (1<<BUTTON)))                //  
		{
			if(button_timer > BUTTON_LDELAY)      //   > 2 
			{
				PORTB &= ~(1<<LED_IN);   	      //  
				PORTB |= (1<<LED_OUT);  	      //  
				work_cnt    = WORK_COUNT;		  //    - 
				cycle_timer = 0;
			}else{
				if(button_timer > BUTTON_SDELAY)  //   < 1 
				{
					work_cnt = 0;
				}
			}
		}else{
			button_timer = 0;
		}
		


		if(work_cnt > 0)                    //   
		{
			if(cycle_timer == 0)            //   
			{
				cycle_timer = TIME_CYCLE;   //   
				
				if(motor_dir)               // 
				{
					Motor_Set (MOTOR_RIGHT);
					motor_dir = 0;
				}else{
					Motor_Set (MOTOR_LEFT);
					motor_dir = 1;
				}
				work_cnt--;                 //  
			}

		}else{								//   
				Motor_Set (MOTOR_STOP);     //
				PORTB &= ~(1<<LED_OUT);     //  
				PORTB |= (1<<LED_IN);       //  	
		}

	}

}


void Motor_Set (unsigned char mode)
{
	motor_mode = mode;             //   
	mprog_select = MOTOR_STOP;     //  
}
void Motor (void) 
{
	
	//    : MOTOR_DELAY
	
	if(motor_timer == 0)  
	{
		switch(mprog_select)
		{
			case MOTOR_START:
				PORTB |= (1<<REL_PWR);    //
			break;
			
			case MOTOR_STOP:
				PORTB &= ~(1<<REL_PWR);   // 
				PORTB &= ~(1<<REL_RVS);   // 
				mprog_select = motor_mode;
			break;
			
			case MOTOR_LEFT:
				PORTB |= (1<<REL_RVS);    //  
				mprog_select = MOTOR_START;
			break;
			
			case MOTOR_RIGHT:
				PORTB &= ~(1<<REL_RVS);   // 
				mprog_select = MOTOR_START;
			break;
		}
		motor_timer = MOTOR_DELAY;
	}else{
		motor_timer--; //   
	}
	
}


ISR (TIM0_COMPA_vect) //   ~100mS
{
	TCNT0 = 0;      // 
	TCCR0B = 0x00;  // 

	Motor();
	
	if(cycle_timer>0)  { cycle_timer--;  }   // 
	button_timer++;                          //   

	TCCR0B = (1<<CS02)|(0<<CS01)|(1<<CS00);  // 
}








